# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import api, fields, models


class ResPartner(models.Model):
    _inherit = "res.partner"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    # Inherit parent
    user_id = fields.Many2one(default=lambda self: self.env.user)

    # New fields
    department_id = fields.Many2one(
        comodel_name="res.department",
        string="Département",
        compute="_compute_department_id",
        store=True,
        readonly=False,
    )
    is_purchasing_centre = fields.Boolean(
        string="Est une centrale d'achat",
        default=False,
    )
    is_direct_invoice = fields.Boolean(
        string="Facturation directe à la centrale",
        default=False,
    )
    purchasing_centre_id = fields.Many2one(
        comodel_name="res.partner",
        string="Centrale d'achat",
        domain=[("is_purchasing_centre", "=", True)],
    )
    membership_number = fields.Char("N° Adhérent")

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("zip", "country_id")
    def _compute_department_id(self):
        for partner in self:
            if partner.zip and partner.country_id == self.env.ref("base.fr"):
                try:
                    zip_dep = partner.zip[:2]
                    if zip_dep == "97":
                        zip_dep = partner.zip[:3]

                    partner.department_id = (
                        self.env["res.department"]
                        .sudo()
                        .search([("code", "=", zip_dep)])
                    )
                except Exception:
                    partner.department_id = False

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------
    @api.onchange("is_purchasing_centre")
    def _onchange_is_purchasing_centre(self):
        if self.is_purchasing_centre:
            self.purchasing_centre_id = False
        if not self.is_purchasing_centre:
            self.is_direct_invoice = False

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def purchasing_centre_customer(self):
        """
        Fonction qui détermine si le contact est client d'une centrale d'achat
        """
        self.ensure_one()
        if self.purchasing_centre_id:
            return self.purchasing_centre_id
        elif self.parent_id and self.parent_id.purchasing_centre_id:
            return self.parent_id.purchasing_centre_id
        else:
            return False
